[Vertex shader]

uniform vec3 lightPos;
uniform vec3 camPos;

varying vec3 lightVec;
varying vec3 viewVec;
varying vec2 texCoord;

void main(){
	gl_Position = ftransform();

	vec3 lVec = lightPos - gl_Vertex.xyz;

	lightVec.x = dot(gl_MultiTexCoord2.xyz, lVec);
	lightVec.y = dot(gl_MultiTexCoord3.xyz, lVec);
	lightVec.z = dot(gl_MultiTexCoord1.xyz, lVec);

	vec3 vVec = camPos - gl_Vertex.xyz;
	viewVec.x = dot(gl_MultiTexCoord2.xyz, vVec);
	viewVec.y = dot(gl_MultiTexCoord3.xyz, vVec);
	viewVec.z = dot(gl_MultiTexCoord1.xyz, vVec);

	texCoord = gl_MultiTexCoord0.xy;
}


[Fragment shader]

uniform sampler2D Base;
uniform sampler2D Bump;

uniform float invRadius;
uniform float factor;
uniform float specular;
uniform vec4 lightColor;

varying vec2 texCoord;
varying vec3 lightVec;
varying vec3 viewVec;

void main(){
	vec4 base = texture2D(Base,  texCoord);
	vec3 bump = texture2D(Bump,  texCoord).xyz * 2.0 - 1.0;

	bump = normalize(bump);

	float distSqr = dot(lightVec, lightVec);
	vec3 lVec = lightVec * inversesqrt(distSqr);

	float atten = clamp(factor * (1.0 - invRadius * invRadius * distSqr), 0.0, 1.0);

	float diffuse = clamp(dot(lVec, bump), 0.0, 1.0);

	float gloss = pow(clamp(dot(reflect(normalize(-viewVec), bump), lVec), 0.0, 1.0), 16.0);

	gl_FragColor = (diffuse * base + gloss * specular) * atten* lightColor;
}
