//============================================================================//
// This source file is part of work done by Reinder Nijhoff (reinder@infi.nl) //
// For the latest info, see http://developer.infi.nl                          //
//                                                                            //
// You're free to use the code in any way you like, modified, unmodified or   //
// cut'n'pasted into your own work.                                           //
//                                                                            //
// Part of this source is based on work by:                                   //
//    - Humus (http://esprit.campus.luth.se/~humus/)                          //
//    - Paul Baker (http://www.paulsprojects.net)                             //
//============================================================================//
#include <stdio.h>
#include "log.hpp"
#include "vector.hpp"
//#include "../gui.hpp"

#define MAX_LENGTH 256

Logger logger;

//==============================================================================
//  Logger:Logger()
//==============================================================================
Logger::Logger() {
    logfile=(FILE* ) NULL;
    
    //Clear the log file contents
    logfile=fopen("log.txt", "wb");
    fclose(logfile);
    
//    m_console = (Overlay_console *)NULL;

/*
    Logstream error; // 0
    Logstream warning; // 1
    Logstream init; // 2
    Logstream function; // 3
    Logstream success; // 4
    Logstream debug; // 5
    Logstream realtime; // 6
*/    

    error.init(    this, 0, true );
    warning.init(  this, 1, true );
    init.init(     this, 2, true );
    function.init( this, 3, true );
    success.init(  this, 4, true );
    debug.init(    this, 5, true );
    realtime.init( this, 6, false );

    m_indent = 0;
    m_log_level = 10;
    
    init << "Logger initialization" << std::endl;
    debug << "by Reinder Nijhoff (reinder@infi.nl)" << std::endl;
    debug << std::endl;
}

//==============================================================================
//  Logger:~Logger()
//==============================================================================
Logger::~Logger() {
    if(logfile) {
        fclose(logfile);
    }    
}

//==============================================================================
//  Logger:output()
//==============================================================================
void Logger::output(bool to_file, int level, char * text) {
    Color m_color;

    if(m_indent<0) m_indent=0;
   
    if( level > m_log_level ) return;
        
    // Write to console
    switch( level) {
        case 5:     m_color = white;
                    break;
        case -1:    m_color = yellow;
                    break;
        case 0:     m_color = green;
                    break;
        case 1:     m_color = orange;
                    break;
        case 2:     m_color = red;
                    break;
        case 3:     m_color = white;
                    break;
        case 4:     m_color = yellow;
                    break;          
    }

/*
    Logstream error; // 0
    Logstream warning; // 1
    Logstream init; // 2
    Logstream function; // 3
    Logstream success; // 4
    Logstream debug; // 5
    Logstream realtime; // 6
*/    
    // Write file
    if(to_file && ((logfile=fopen("log.txt", "a+"))!=NULL)) {
        switch( level) {
            case 0:  fprintf(logfile, "<!> "); // function
                     break;
            case 1:  fprintf(logfile, "(!) "); // succes                    
                     break;
            case 2:  fprintf(logfile, "(+) "); // warning
                     break;
            case 3:  fprintf(logfile, "(+) "); // error
                     break;
            case 4:  fprintf(logfile, "(-) "); // debug
                     break;
            case 5:  fprintf(logfile, "( ) "); // function
                     break;
        }

        for(int i=0; i<m_indent*2; ++i) {
               putc(' ', logfile);
        }    
        
        fprintf(logfile, text);
        putc('\n', logfile);    
        fclose(logfile);
    }
    
    //Write to console
/*    if(m_console!=NULL) {
        m_console->text()->print(m_color, temp);
        if(m_console->get_direct_render()) m_console->direct_render();
    }*/
}

//==============================================================================
//  Logstreambuf::overflow()
//==============================================================================
int Logstreambuf::overflow(int c) {
    if (c == EOF || c == '\n') {
        m_logger->output(m_to_file, m_level, (char *)lnbuffer.c_str() );
        if(m_level == 3) {
            m_logger->begin_function();
        }    
        if( (m_level == 0) || (m_level == 4) ) {
            m_logger->end_function();
        }    
        lnbuffer = "";
    } else {
        lnbuffer += (char) c;
    }    
    return 1;
} 

//==============================================================================
//  set_console()
//==============================================================================
/*void Logger::set_console(Overlay_console* n_console) {
    m_console = n_console;
}*/

//==============================================================================
//  Logstream::init()
//==============================================================================
void Logstream::init(Logger* n_logger, int n_level, bool to_file) {
    m_buffer.init(n_logger, n_level, to_file);
}

//==============================================================================
//  Logstreambuf::init()
//==============================================================================
void Logstreambuf::init(Logger* n_logger, int n_level, bool to_file) {
   m_logger = n_logger;
   m_level = n_level;
   m_to_file = to_file;
}    

