//============================================================================//
// This source file is part of work done by Reinder Nijhoff (reinder@infi.nl) //
// For the latest info, see http://developer.infi.nl                          //
//                                                                            //
// You're free to use the code in any way you like, modified, unmodified or   //
// cut'n'pasted into your own work.                                           //
//                                                                            //
// Part of this source is based on work by:                                   //
//    - Humus (http://esprit.campus.luth.se/~humus/)                          //
//    - Paul Baker (http://www.paulsprojects.net)                             //
//============================================================================//

#ifndef  _MATERIAL_
#define  _MATERIAL_ 1

#include <vector>
#include <string>

#include "includes.hpp"
#include "vector.hpp"
#include "data.hpp"
#include "image.hpp"
#include "texture.hpp"

class Image;
class Texture;

//==============================================================================
//  Material_value
//==============================================================================
class Material_value {
public:
	Material_value() { m_type = UNDEFINED; }
	~Material_value() {}	    
	  
    enum Value_type {
        UNDEFINED = 0,
        TEXTURE = 1,
        COLOR = 2,
        FLOAT = 3
    };
	  
	void set_texture( Texture* texture )	{ m_type = TEXTURE; m_value.m_texture =  texture; }
	void set_color( const Color& color) 	{ m_type = COLOR;   m_color = color; }
	void set_float( float val)				{ m_type = FLOAT;   m_value.m_float = val; }
		
	Color get_color( float u, float v, int img_index ) {
		if( m_type == TEXTURE ) {
		    Image* image = m_value.m_texture->get_image(img_index);
			return image->get_pixel<Color>( u, v, m_value.m_texture->get_use_clamp() );
		} else  {
	        return m_color;   
		}
    }

	float get_float( float u, float v, int img_index ) {
        if( m_type == TEXTURE ) {
            Image* image = m_value.m_texture->get_image(img_index);
  		    return image->get_pixel<float>( u, v, m_value.m_texture->get_use_clamp() );
	    } else  {
	  	    return m_value.m_float;
	    }
	}
	
	Texture* get_texture() { return m_value.m_texture; }
	
	bool used() { return m_type != UNDEFINED; }
	void clear() { m_type = UNDEFINED; }
private:		
	Value_type m_type;		
	union {	
	    Texture* m_texture;
		float	 m_float;
	} m_value;
	Color m_color;
};

//==============================================================================
//  Material
//==============================================================================
class Material {
public:
	Material();
    ~Material();
    
    virtual void init(Data_node* data);    
    void clear();
  
    enum Material_layer_type {
        DIFFUSE = 0,
        BUMP = 1,
        SPECULAR = 2,
        REFRACTION_INDEX = 3,
        TRANSPARENCY = 4,
        EMISSION = 5,
		NUM_OF_TYPES = 6   
    };

    void set_color( int index, const Color& color ) { m_values[index].set_color( color ); }
    void set_texture( int index, Texture* tex )		{ m_values[index].set_texture( tex ); }
    void set_float( int index, float value )		{ m_values[index].set_float( value ); }
    
    Color get_color( int index, const Vector3f& direction );       	 
    inline Color get_color( int index, float u=0.0f, float v=0.0f, int img_index=0 ) {
		return m_values[index].get_color( u*m_tex_scale, v*m_tex_scale, img_index );  
    }    
    inline float get_float( int index, float u=0.0f, float v=0.0f, int img_index=0 ) {
		return m_values[index].get_float( u*m_tex_scale, v*m_tex_scale, img_index );  
    }
    inline Texture* get_texture(int index ) { return m_values[index].get_texture(); }
    
    bool used( int index ) { return m_values[index].used(); }
public:
    std::vector<Material_value> m_values;
    
    float m_tex_scale;
};

#endif /* MATERIAL */

