//============================================================================//
// This source file is part of work done by Reinder Nijhoff (reinder@infi.nl) //
// For the latest info, see http://developer.infi.nl                          //
//                                                                            //
// You're free to use the code in any way you like, modified, unmodified or   //
// cut'n'pasted into your own work.                                           //
//                                                                            //
// Part of this source is based on work by:                                   //
//    - Humus (http://esprit.campus.luth.se/~humus/)                          //
//    - Paul Baker (http://www.paulsprojects.net)                             //
//============================================================================//

#ifndef _PORTAL_SCENE_
#define _PORTAL_SCENE_ 1

#include "scene.hpp"
#include "batch.hpp"
#include "vector.hpp"
#include "matrix.hpp"
#include "geometry.hpp"
#include "renderer.hpp"
#include "texture.hpp"

#include <string>
#include <algorithm>

class Portal_portal;
class Scene_portal;
class Camera;

//==============================================================================
//  Doom3_node
//==============================================================================
struct Doom3_node {
    Plane plane;
    int pos_child, neg_child;
};

//==============================================================================
//  Vector2i
//==============================================================================
class Vector2i {
public:
    Vector2i() {}
    Vector2i(int n_x, int n_y) : x(n_x), y(n_y) {}
    
    int x, y;
};

//==============================================================================
//  Portal_area
//==============================================================================
class Portal_area {
	public:
		Portal_area(Scene_portal* scene, const std::string & name, int index) : m_name(name), m_index(index) {
			m_scene = scene;						  
		}
		const std::string & get_name() const { return m_name; }

		void render_from_portal(Camera* camera, Portal_portal* portal, Vector2i min, Vector2i max);

		void add_portal(    Portal_portal *portal) { m_portals.push_back( portal ); }
		void add_light( 	Primitive *light ) {
			if( std::find(m_lights.begin(), m_lights.end(), light ) == m_lights.end() ) {  
				m_lights.push_back( light ); 
			} 
		}
		void add_primitive( Primitive *prim  ) {
			if( find(m_primitives.begin(), m_primitives.end(), prim ) == m_primitives.end() ) {  
				m_primitives.push_back( prim ); 
			}
		}

    void read_from_file(std::ifstream &file);
private:
    Scene_portal* m_scene;
    std::string m_name;
    int m_index;
    
    std::vector<Primitive *> m_primitives;
    std::vector<Primitive *> m_lights;
    std::vector<Portal_portal *> m_portals;
       
	std::map<Portal_portal*, int> m_rendered;   
};

//==============================================================================
//  Portal_portal
//==============================================================================
class Portal_portal {
public:
    Portal_portal(Scene_portal* scene) : m_frame_rendered(999) { m_scene = scene; }
     
    void render_from_area(Camera* camera, int index, Vector2i min, Vector2i max);    
      
    void read_from_file(std::ifstream &file);
    
    int  check_visibility(Camera* camera);
    void transform_points();
private:
    Scene_portal* m_scene;
    
    Vector2i m_transformed_min;
    Vector2i m_transformed_max;
    
    std::vector<Vector3f> m_points;
    std::vector<Vector2i> m_transformed_points;
    int m_area_pos;
    int m_area_neg;
    
    int m_frame_rendered;
    int m_visible;
};
    
//==============================================================================
//  Scene_portal
//==============================================================================
class Scene_portal : public Scene {
public:    
    Scene_portal() : m_use_portals(true), m_debug_light_quad(false), 
				     m_debug_light_aa_box(false), m_shader_index(0) {}
    
    virtual void init(Data_node* data);
	bool update();	
    void render();

    void load_proc(const std::string & name);
    void load_map(const std::string & name);
    void load_mtr(const std::string & name);
    
    Portal_area * get_area(int i) { return m_areas[i]; }    
    int get_area(const Vector3f & position);
    
    int get_area_index_by_name(const std::string & name);
    void push_light(Primitive* light, Doom3_node *node = NULL);
    
    bool get_debug_light_quad()   { return m_debug_light_quad; }
    bool get_debug_light_aa_box() { return m_debug_light_aa_box; }

    bool set_debug_light_quad(bool debug)   { m_debug_light_quad = debug; }
    bool set_debug_light_aa_box(bool debug) { m_debug_light_aa_box = debug; }	
	
	Shader* get_shader() { return m_shader; }
private:    
    std::vector<Portal_area *> m_areas;
    std::vector<Portal_portal *> m_portals;
    std::vector<Doom3_node> m_nodes;    
    
    Shader* m_shader;
    int m_shader_index;
    bool m_use_portals, m_debug_light_quad, m_debug_light_aa_box;
};

#endif /* _PORTAL_SCENE_ */
