//============================================================================//
// This source file is part of work done by Reinder Nijhoff (reinder@infi.nl) //
// For the latest info, see http://developer.infi.nl                          //
//                                                                            //
// You're free to use the code in any way you like, modified, unmodified or   //
// cut'n'pasted into your own work.                                           //
//                                                                            //
// Part of this source is based on work by:                                   //
//    - Humus (http://esprit.campus.luth.se/~humus/)                          //
//    - Paul Baker (http://www.paulsprojects.net)                             //
//============================================================================//
#include <stdio.h>
#include "shader.hpp"
#include "material.hpp"

#include "renderer.hpp"
#include "base.hpp"
#include "camera.hpp"
#include "primitive.hpp"
#include "factory.hpp"

//==============================================================================
//  Register to Factory
//==============================================================================
bool fac_shader  = Factory_shader.reg<Shader>( "shader" );

//==============================================================================
//  Shader_value::set_value()
//==============================================================================
void Shader_value::set_value(	float value ) {
	m_value.x = value; 
}

//==============================================================================
//  Shader_value::set_value()
//==============================================================================
void Shader_value::set_value(	Vector2f value ) {
	m_value.x = value.x;
	m_value.y = value.y;	
}

//==============================================================================
//  Shader_value::set_value()
//==============================================================================
void Shader_value::set_value(	Vector3f value )  {
	m_value.x = value.x;
	m_value.y = value.y;	
	m_value.z = value.z;	
}

//==============================================================================
//  Shader_value::set_value()
//==============================================================================
void Shader_value::set_value(	Vector4f value )  {
	m_value = value;
}

//==============================================================================
//  Shader::init()
//==============================================================================
void Shader::init(Data_node* data) {
	clear(); 
	
	std::string s, t, n, d;
	float f;
	Vector2f v2;
	Vector3f v3;
	Vector4f v4;
	
    data->reset();
    
    while( Data_value* value = data->get_next_value() ) {
	    s = value->get_key();
	    if( s == "material_constant" ) {
			Shader_value* s_value = new Shader_value();
			value->read( t );
			value->read( n );
			s_value->set_name( n );
			
			if( t == "MATERIAL_FLOAT" ) { 
				s_value->set_type( Shader_value::MATERIAL_FLOAT ); 
				value->read( f ); 
				s_value->set_value( f );
			}
			if( t == "MATERIAL_VECTOR2F" )  { 
				s_value->set_type( Shader_value::MATERIAL_VECTOR2F ); 
				value->read( v2 ); 
				s_value->set_value( v2 );
			}
			if( t == "MATERIAL_VECTOR3F" )  { 
				s_value->set_type( Shader_value::MATERIAL_VECTOR3F ); 
				value->read( v3 ); 
				s_value->set_value( v3 );
			}
			if( t == "MATERIAL_VECTOR4F" )  { 
				s_value->set_type( Shader_value::MATERIAL_VECTOR4F ); 
				value->read( v4 ); 
				s_value->set_value( v4 );
			}
			if( t == "MATERIAL_TEXTURE" )  { 
				s_value->set_type( Shader_value::MATERIAL_TEXTURE ); 
				int m_index = 0;
				value->read( m_index );
				s_value->set_material_index( m_index );
			}
			m_constants.push_back( s_value );
			m_material_constants.push_back( s_value );
		} else if( s == "light_constant" ) {
			Shader_value* s_value = new Shader_value();
			value->read( t );
			value->read( n );
			s_value->set_name( n );
						
			if( t == "LIGHT_POS" ) { 
				s_value->set_type( Shader_value::LIGHT_POS ); 
			}
			if( t == "LIGHT_INV_RADIUS" ) { 
				s_value->set_type( Shader_value::LIGHT_INV_RADIUS ); 
			}
			if( t == "LIGHT_RADIUS" ) { 
				s_value->set_type( Shader_value::LIGHT_RADIUS ); 
			}									
			if( t == "LIGHT_COLOR" ) { 
				s_value->set_type( Shader_value::LIGHT_COLOR ); 
			}						
			m_constants.push_back( s_value );
			m_light_constants.push_back( s_value );
  		} else if( s == "global_constant" ) {
			Shader_value* s_value = new Shader_value();
			value->read( t );
			value->read( n );
			s_value->set_name( n );
						
			if( t == "CAMERA_POS" ) { 
				s_value->set_type( Shader_value::CAMERA_POS ); 
			}
			if( t == "TIME" ) { 
				s_value->set_type( Shader_value::TIME ); 
			}
			
			m_constants.push_back( s_value );
			m_global_constants.push_back( s_value );
		} else if( s == "shader_constant" ) {
			Shader_value* s_value = new Shader_value();
			value->read( t );
			value->read( n );
			s_value->set_name( n );
						
			if( t == "FLOAT" ) { 
				s_value->set_type( Shader_value::FLOAT ); 
				value->read( f ); 
				s_value->set_value( f );
			}
			if( t == "VECTOR2F" )  { 
				s_value->set_type( Shader_value::VECTOR2F ); 
				value->read( v2 ); 
				s_value->set_value( v2 );
			}
			if( t == "VECTOR3F" )  { 
				s_value->set_type( Shader_value::VECTOR3F ); 
				value->read( v3 ); 
				s_value->set_value( v3 );
			}
			if( t == "VECTOR4F" )  { 
				s_value->set_type( Shader_value::VECTOR4F ); 
				value->read( v4 ); 
				s_value->set_value( v4 );
			}
			m_constants.push_back( s_value );
			m_shader_constants.push_back( s_value );   
  		}
	}
}

//==============================================================================
//  Shader::clear()
//==============================================================================
void Shader::clear() {
	m_constants.resize(0);
	m_material_constants.resize(0);
	m_global_constants.resize(0);
	m_light_constants.resize(0); 
}

//==============================================================================
//  Shader::first_update()
//==============================================================================
void Shader::first_update() {
	renderer->set_shader( this );
// bind all shader values...
    for(int i=0, l=m_constants.size(); i<l; ++i) {
		int index = bind_value( *m_constants[i] );
		if( index < 0 ) {
			log_fatal << "Unable to bind shader value " << m_constants[i]->get_name() << std::endl;
		} 
		m_constants_index[ m_constants[i]->get_name() ] = index;
		m_constants[i]->set_index( index );
	}	 
}

//==============================================================================
//  Shader::set_shader_global_constants()
//==============================================================================	 
void Shader::set_shader_global_constants() {
	for(int i=0, l=m_global_constants.size(); i<l; ++i) {
		int index = m_global_constants[i]->get_index();
		switch( m_global_constants[i]->get_type() ) {
			case Shader_value::CAMERA_POS:		set_shader_constant( index, renderer->get_camera()->get_position() );
				 								break;
			case Shader_value::TIME:			set_shader_constant( index, base->get_sys_time());
				 								break;								 	
		}
	} 
}

//==============================================================================
//  Shader::set_shader_constants()
//==============================================================================
void Shader::set_shader_constants() {
	for(int i=0, l=m_shader_constants.size(); i<l; ++i) {
		int index = m_shader_constants[i]->get_index();
		switch( m_shader_constants[i]->get_type() ) {
			case Shader_value::FLOAT:			set_shader_constant( index, *(float *) m_shader_constants[i]->get_value() );
				 								break;
			case Shader_value::VECTOR2F:		set_shader_constant( index, *(Vector2f *) m_shader_constants[i]->get_value() );
				 								break;
			case Shader_value::VECTOR3F:		set_shader_constant( index, *(Vector3f *) m_shader_constants[i]->get_value() );
				 								break;
			case Shader_value::VECTOR4F:		set_shader_constant( index, *(Vector4f *) m_shader_constants[i]->get_value() );
				 								break;									 				 					
		}
	} 
}
   
//==============================================================================
//  Shader::set_shader_material_constants()
//==============================================================================
void Shader::set_shader_material_constants(Material *material) {
	Vector3f temp3f;
	Vector4f temp4f;
	
	int texture_slot = 0;
	 
	for(int i=0, l=m_material_constants.size(); i<l; ++i) {
		int index = m_material_constants[i]->get_index();
		int material_index = m_material_constants[i]->get_material_index();
		switch( m_material_constants[i]->get_type() ) {
			case Shader_value::MATERIAL_FLOAT:		set_shader_constant( index, material->get_float( material_index ) );
				 									break;				 									
			case Shader_value::MATERIAL_VECTOR3F:	temp3f = material->get_color( material_index );
		 		 									set_shader_constant( index, temp3f );
				 									break;
			case Shader_value::MATERIAL_VECTOR4F:	temp4f = Vector4f( material->get_color( material_index ) );
	  			 									set_shader_constant( index, temp4f );
				 									break;
			case Shader_value::MATERIAL_TEXTURE:	renderer->bind_texture( texture_slot, material->get_texture( material_index ) );
	  			 									set_shader_constant( index, texture_slot );
	  			 									texture_slot++;
				 									break;				 									
		}
	} 
}

//==============================================================================
//  Shader::set_shader_light_constants()
//==============================================================================	
void Shader::set_shader_light_constants(Primitive * light) {
	Vector4f temp4f; 
	for(int i=0, l=m_light_constants.size(); i<l; ++i) {
		int index = m_light_constants[i]->get_index();
		switch( m_light_constants[i]->get_type() ) {
			case Shader_value::LIGHT_POS:			set_shader_constant( index, light->get_position() );
				 									break;
			case Shader_value::LIGHT_INV_RADIUS:	set_shader_constant( index, 1.0f/light->get_intensity() );
				 									break;
			case Shader_value::LIGHT_RADIUS:		set_shader_constant( index, light->get_intensity() );
				 									break;	
			case Shader_value::LIGHT_COLOR:			temp4f = Vector4f( light->get_material()->get_color( Material::DIFFUSE ) );
	  			 									set_shader_constant( index, temp4f );
				 									break;										 			 					
		}
	} 
}

//==============================================================================
//  Shader::read_from_file()
//==============================================================================
void Shader::read_from_file(const std::string& name){
	FILE *file = fopen(name.c_str(), "rb");
	if (file == NULL){
		log_warning << "Couldn't load \"" << name << "\"" << std::endl;
	} else {
		// Find file size
		fseek(file,  0, SEEK_END);
		int length = ftell(file);
		fseek(file,  0, SEEK_SET);

		char *shader_text = new char[length + 1];
		fread(shader_text, length, 1, file);
		fclose(file);
		shader_text[length] = '\0';

		char *vertex_shader   = strstr(shader_text, "[Vertex shader]");
		char *fragment_shader = strstr(shader_text, "[Fragment shader]");

		if (vertex_shader != NULL){
			*vertex_shader = '\0';
			vertex_shader += 15;
		}
		if (fragment_shader != NULL){
			*fragment_shader = '\0';
			fragment_shader += 17;
		}

		upload_shader(vertex_shader, fragment_shader);

		delete [] shader_text;
	}
}
